﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Office.Interop.Visio;

// VisioDrawer.cs
// Code for drawing the internal representation of the Hadoop cluster to Visio

namespace HadoopVisProto
{
    class VisioDrawer
    {
        private static Application VisioApp;
        private static Window ActiveWindow;
        private static Document ActiveDoc;
        private static Page ActivePage;

        private static Document networkTemplate = null;
        private static Master masterNode;
        private static Master slaveNode;
        private static Master rack;
        private static Master dynamicConnector;
        
        private static Document slaveBase = null;
        private static Master slaveDataMaster;

        public static void quit()
        {
            // Kill app if Visio exits
            VisioApp.Quit();
        }

        public static void createPage() 
        {
            // Start application and open new document
            VisioApp = new Application();
            VisioApp.BeforeDocumentClose += new EApplication_BeforeDocumentCloseEventHandler(quit);
            ActiveWindow = VisioApp.ActiveWindow;
            ActiveDoc = VisioApp.Documents.Add("");
            ActivePage = ActiveDoc.Pages.Add();
            ActivePage.AutoSize = true;

            // Open visual templates
            networkTemplate = VisioApp.Documents.OpenEx(@"B:\Projects\HadoopVis\HadoopVisProto\NetTemplate.vstx",
                    (short)VisOpenSaveArgs.visOpenHidden);
            Document pcStencil = VisioApp.Documents["COMPME_M.VSSX"];
            Document networkStencil = VisioApp.Documents["PERIME_M.VSSX"];
            Shape PageInfo = ActivePage.PageSheet;
            PageInfo.get_CellsSRC((short)VisSectionIndices.visSectionObject, (short)VisRowIndices.visRowPageLayout, (short)(VisCellIndices.visPLOPlaceStyle)).set_Result(VisUnitCodes.visPageUnits, (double)VisCellVals.visPLOPlaceTopToBottom);
            PageInfo.get_CellsSRC((short)VisSectionIndices.visSectionObject, (short)VisRowIndices.visRowPageLayout, (short)(VisCellIndices.visPLORouteStyle)).set_Result(VisUnitCodes.visPageUnits, (double)VisCellVals.visLORouteFlowchartNS);
            ActivePage.SetTheme("Whisp");

            // Get all the master shapes
            masterNode = pcStencil.Masters.get_ItemU("PC");
            slaveNode = networkStencil.Masters.get_ItemU("Server");
            rack = networkStencil.Masters.get_ItemU("Switch");
            dynamicConnector = networkStencil.Masters.get_ItemU("Dynamic connector");

            // Open data visualization template and shape
            slaveBase = VisioApp.Documents.OpenEx(@"B:\Projects\HadoopVis\HadoopVisProto\DataGraphicTemplate.vsdx",
                    (short)Microsoft.Office.Interop.Visio.VisOpenSaveArgs.visOpenHidden);
            slaveDataMaster = slaveBase.Pages[1].Shapes[1].DataGraphic;
        }

        private static void quit(Document doc)
        {
            VisualizerMain.closedFlag = true; 
        }

        public static void drawTree(HadoopNode jobTracker, HadoopNode nameNode, Dictionary<string, HadoopNode> innerNodes, Dictionary<string, HadoopNode> leaves)
        {
            // Draw JobTracker
            Shape jtShape = jobTracker.setShape(ActivePage.Drop(masterNode, 3, 8));
            jtShape.Text = "JobTracker" + "(" + jobTracker.name + ")";

            // Draw NameNode
            Shape nnShape = nameNode.setShape(ActivePage.Drop(masterNode, 6, 8));
            nnShape.Text = "NameNode" + "(" + nameNode.name + ")";

            // Draw all racks
            foreach (KeyValuePair<string, HadoopNode> r in innerNodes)
            {
                HadoopNode rackNode = r.Value;
                Shape rShape;
                if (rackNode.getHDFSParent() != null && rackNode.getMapredParent() != null && rackNode.getHDFSParent() == rackNode.getMapredParent()) 
                {
                    rShape = rackNode.setShape(ActivePage.DropConnected(rack, rackNode.getHDFSParent().getShape(), VisAutoConnectDir.visAutoConnectDirDown, dynamicConnector));
                }
                else if (rackNode.getHDFSParent() != null)
                {
                    rShape = rackNode.setShape(ActivePage.DropConnected(rack, rackNode.getHDFSParent().getShape(), VisAutoConnectDir.visAutoConnectDirDown, dynamicConnector));
                    if (rackNode.getMapredParent() != null)
                    {
                        rackNode.getMapredParent().getShape().AutoConnect(rShape, VisAutoConnectDir.visAutoConnectDirDown, dynamicConnector);
                    }
                }
                else
                {
                    rShape = rackNode.setShape(ActivePage.DropConnected(rack, rackNode.getMapredParent().getShape(), VisAutoConnectDir.visAutoConnectDirDown, dynamicConnector));
                }
                rShape.Text = rackNode.name;
            }

            // Draw all slave nodes
            foreach (KeyValuePair<string, HadoopNode> s in leaves)
            {
                HadoopNode sNode = s.Value;
                Shape sShape;
                if (sNode.getHDFSParent() != null)
                {
                    sShape = sNode.setShape(ActivePage.DropConnected(slaveNode, sNode.getHDFSParent().getShape(), VisAutoConnectDir.visAutoConnectDirDown, dynamicConnector));
                }
                else
                {
                    sShape = sNode.setShape(ActivePage.DropConnected(slaveNode, sNode.getMapredParent().getShape(), VisAutoConnectDir.visAutoConnectDirDown, dynamicConnector));
                }
                sShape.Text = sNode.name;
                sShape.DataGraphic = slaveDataMaster;
                
            }
            ActivePage.Layout();
            ActivePage.AutoSizeDrawing();
            //ActivePage.LayoutIncremental(VisLayoutIncrementalType.visLayoutIncrAlign, VisLayoutHorzAlignType.visLayoutHorzAlignDefault, VisLayoutVertAlignType.visLayoutVertAlignNone, 1, 1, VisUnitCodes.visMillimeters);
            //ActivePage.LayoutIncremental(VisLayoutIncrementalType.visLayoutIncrSpace, VisLayoutHorzAlignType.visLayoutHorzAlignRight, VisLayoutVertAlignType.visLayoutVertAlignNone, 1, 1, VisUnitCodes.visMillimeters);
        }

        public static void updateData(Dictionary<string, HadoopNode> leaves)
        {
            foreach (KeyValuePair<string, HadoopNode> l in leaves)
            {
                HadoopNode leaf = l.Value;
                Shape leafShape = leaf.getShape();
                // Update HDFS information
                if (leaf.dataActive)
                {
                    leafShape.get_CellsSRC(243, 20, 0).set_Result(0, leaf.getHDSpace());
                    leafShape.get_CellsSRC(243, 0, 0).set_Result(0, 1);
                }
                // If the DataNode has failed, turn the bottom checkmark to a red X
                else
                {
                    leafShape.get_CellsSRC(243, 0, 0).set_Result(0, 0);
                }
                // Update mapred information
                if (leaf.taskActive)
                {
                    leafShape.get_CellsSRC(243, 17, 0).set_Result(0, leaf.getMapUse());
                    leafShape.get_CellsSRC(243, 18, 0).set_Result(0, leaf.getReduceUse());
                    leafShape.get_CellsSRC(243, 12, 0).set_Result(0, 1);
                }
                // If the Tasktracker has failed, turn the bottom checkmark to a red X
                else
                {
                    leafShape.get_CellsSRC(243, 12, 0).set_Result(0, 0);
                }
            }
        }
    }
}
